<?php

/* let PHP run just as long as it has to */
ini_set("max_execution_time", "0");

error_reporting('E_ALL');
$dir = dirname(__FILE__);
chdir($dir);

if (strpos($dir, 'plugins') !== false) {
	chdir('../../');
}

include("./include/config.php");
include_once($config["base_path"] . "/lib/snmp.php");
include_once($config["base_path"] . "/lib/ping.php");

/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);

$debug = FALSE;
$forcerun = FALSE;

foreach($parms as $parameter) {
	@list($arg, $value) = @explode("=", $parameter);

	switch ($arg) {
	case "-r":
		discover_recreate_tables();
		break;
	case "-d":
		$debug = TRUE;
		break;
	case "-h":
		display_help();
		exit;
	case "-f":
		$forcerun = TRUE;
		break;
	case "-v":
		display_help();
		exit;
	case "--version":
		display_help();
		exit;
	case "--help":
		display_help();
		exit;
	default:
		print "ERROR: Invalid Parameter " . $parameter . "\n\n";
		display_help();
		exit;
	}
}

function discover_recreate_tables () {
	discover_debug("Request received to recreate the Discover Plugin's tables\n");
	discover_debug("   Dropping the tables\n");
	mysql_query("drop table plugin_discover_hosts");
	mysql_query("drop table plugin_discover_os");

	discover_debug("   Creating the tables\n");
	discovery_setup_table ();
}


if (read_config_option("discovery_collection_timing") == "disabled") {
	discover_debug("Discovery Polling is set to disabled.\n");
	exit;
}

discover_debug("Checking to determine if it's time to run.\n");

$seconds_offset = read_config_option("discovery_collection_timing");
$seconds_offset = $seconds_offset * 60;
$base_start_time = read_config_option("discovery_base_time");
$last_run_time = read_config_option("discovery_last_run_time");
$previous_base_start_time = read_config_option("discovery_prev_base_time");

if ($base_start_time == '') {
	discover_debug("Base Starting Time is blank, using '12:00am'\n");
	$base_start_time = '12:00am';
}

/* see if the user desires a new start time */
discover_debug("Checking if user changed the start time\n");
if (!empty($previous_base_start_time)) {
	if ($base_start_time <> $previous_base_start_time) {
		discover_debug("   User changed the start time from '$previous_base_start_time' to '$base_start_time'\n");
		unset($last_run_time);
		db_execute("DELETE FROM settings WHERE name='discovery_last_run_time'");
	}
}

/* set to detect if the user cleared the time between polling cycles */
db_execute("REPLACE INTO settings (name, value) VALUES ('discovery_prev_base_time', '$base_start_time')");

/* determine the next start time */
$current_time = strtotime("now");
if (empty($last_run_time)) {
	if ($current_time > strtotime($base_start_time)) {
		/* if timer expired within a polling interval, then poll */
		if (($current_time - 300) < strtotime($base_start_time)) {
			$next_run_time = strtotime(date("Y-m-d") . " " . $base_start_time);
		}else{
			$next_run_time = strtotime(date("Y-m-d") . " " . $base_start_time) + 3600*24;
		}
	}else{
		$next_run_time = strtotime(date("Y-m-d") . " " . $base_start_time);
	}
}else{
	$next_run_time = $last_run_time + $seconds_offset;
}
$time_till_next_run = $next_run_time - $current_time;

if ($time_till_next_run < 0) {
	discover_debug("The next run time has been determined to be NOW\n");
}else{
	discover_debug("The next run time has been determined to be at\n   " . date("Y-m-d G:i:s", $next_run_time) . "\n");
}

if ($time_till_next_run > 0 && $forcerun == FALSE)
	exit;

if ($forcerun)
	discover_debug("Scanning has been forced\n");

if ($forcerun == FALSE)
	db_execute("REPLACE INTO settings (name, value) VALUES ('discovery_last_run_time', '$current_time')");








cacti_log("Network Discover is now running", true, "POLLER");

// Get an array of our current hostnames
$temp = db_fetch_assoc("SELECT hostname FROM host");
$known = array();
foreach ($temp as $d) {
	$known[] = $d['hostname'];
}

// Get Oses
$temp = db_fetch_assoc("SELECT * FROM plugin_discover_os");
$os = array();
if (is_array($temp)) {
	foreach ($temp as $d) {
		$os[] = $d;
	}
}

$dns = trim(read_config_option("discovery_dns"));

$ping_type = read_config_option("discovery_protocol");
if ($ping_type < 0 || $ping_type > 2)
	$ping_type = 0;

switch ($ping_type) {
	case 2:
		$method = PING_ICMP;
		break;
	case 1:
		$method = PING_TCP;
		break;
	case 0:
		$method = PING_UDP;
		break;
}


// This is our current hosts

$hline = trim(read_config_option("discovery_subnet"));
if ($hline == '') {
	cacti_log("ERROR: Network Discover subnet setting is not set!", true, "POLLER");
	return;
}

$hline = str_replace(' ', '', $hline);
$hosts = explode(',', $hline);

$arg = 0;

$t = time() - 3600;
db_execute("delete from plugin_discover_hosts where time < $t");

db_execute("update plugin_discover_hosts set up = 0");

$cnames = read_config_option("discovery_readstrings");

if ($cnames == '') {
	$pcname = 'public';
	$scname = '';
} else {
	$cnames = explode(':',$cnames);
	$pcname = $cnames[0];
	unset($cnames[0]);
	$scname = implode(':',$cnames);
}

discover_debug("Primary Community Name    : $pcname\n");
discover_debug("Secondary Community Names : $scname\n");

for ($arg = 0; $arg < count($hosts); $arg++) {
	$count = 0;

	discover_debug("Calculating Info for Subnet :" . $hosts[$arg] . "\n");
	$start = discover_calculate_start($hosts, $arg);
	$total = discover_calculate_total_ips($hosts, $arg);

	discover_debug("Start IP is " . $start . "\n");
	discover_debug("Total IPs is " . $total . "\n");

	while ($host = discover_get_next_host($start, $total, $count)) {
		$count++;
		discover_debug($host);
		if (!in_array($host, $known)) {
			if (substr($host, -3) < 255) {
				$hash = discover_ip_hash($host);
				$device = array(
					'snmp_status'		=> 0,
					'ping_status'		=> 0,
					'hostname'		=> $host,
					'community'		=> '',
					'dnsname'		=> '',
					'snmp_readstring'	=> $pcname,
					'snmp_readstrings'	=> $scname,
					'snmp_version'	=> 1,
					'snmp_port'		=> 161,
					'snmp_timeout'	=> 100,
					'snmp_sysObjectID'	=> '',
					'snmp_sysName'	=> '',
					'snmp_sysLocation'	=> '',
					'snmp_sysContact'	=> '',
					'snmp_sysDescr'	=> '',
					'snmp_sysUptime'	=> 0,
					'os'			=> '',
					'hash' => $hash);

				if (discover_valid_snmp_device($device)){
					if ($dns != '') {
						$dnsname = discover_get_dns_from_ip($host, $dns, 300);
						if ($dnsname != $host && $dnsname != 'timed_out')
							$device['dnsname'] = $dnsname;
					}
					if ($dns != '' && in_array($device['dnsname'], $known)) {
						discover_debug(" - Host DNS is already in hosts table!");
					} else {
						discover_debug(" is a valid device!");
						$fos = discover_find_os($device['snmp_sysDescr']);
						if ($fos != false) {
							$device['os'] = $fos;
						}
						db_execute("REPLACE INTO plugin_discover_hosts (hostname, ip, hash, community, sysName, sysLocation, sysContact, sysDescr, sysUptime, os, snmp, up, time) VALUES ('"
							. sql_sanitize($device['dnsname'])
							. "', '$host', '"
							. $device['hash'] . "', '"
							. sql_sanitize($device['community']) . "', '"
							. sql_sanitize($device['snmp_sysName']) . "', '"
							. sql_sanitize($device['snmp_sysLocation']) . "', '"
							. sql_sanitize($device['snmp_sysContact']) . "', '"
							. sql_sanitize($device['snmp_sysDescr']) . "', '"
							. sql_sanitize($device['snmp_sysUptime']) . "', '"
							. sql_sanitize($device['os']) . "', "
							. "1, 1,".time() . ')' );
					}
				} else {
					if (function_exists("socket_create") && phpversion() > "4.3") {
						$ping = new Net_Ping;
						$ping->host["hostname"] = $host;
						$ping->retries = 1;
						$ping->port = 0;
						$result = $ping->ping(AVAIL_PING, $method, $device['snmp_timeout'], 1);

						if ($result) {
							if ($dns != '') {
								$dnsname = discover_get_dns_from_ip($host, $dns, 300);
								if ($dnsname != $host && $dnsname != 'timed_out')
									$device['dnsname'] = $dnsname;
							}
							db_execute("REPLACE INTO plugin_discover_hosts (hostname, ip, hash, community, sysName, sysLocation, sysContact, sysDescr, sysUptime, os, snmp, up, time) VALUES ('"
								. sql_sanitize($device['dnsname'])
								. "', '$host', '"
								. $device['hash'] . "', '"
								. sql_sanitize($device['community']) . "', '"
								. sql_sanitize($device['snmp_sysName']) . "', '"
								. sql_sanitize($device['snmp_sysLocation']) . "', '"
								. sql_sanitize($device['snmp_sysContact']) . "', '"
								. sql_sanitize($device['snmp_sysDescr']) . "', "
								. sql_sanitize($device['snmp_sysUptime']) . ", "
								. "'', 0, 1,".time() . ')' );
							discover_debug(" - Host is alive but no SNMP!");
						} else {
						}
					}
				}
			} else {
				discover_debug(" - Ignoring Address!");
			}
		} else {
			discover_debug(" - Host is already in hosts table!");
		}
		discover_debug("\n");
	}
}

function discover_ip_hash($ip) {
	$ips = explode('.',$ip);
	$hash = ($ips[0] * 16777216) + ($ips[1] * 65536) + ($ips[2] * 256) + $ips[3];
	return $hash;
}

function discover_find_os($text) {
	global $os;
	for ($a = 0; $a < count($os); $a++) {
		$s = '';
		if (isset($os[$a]['matchstring']))
			$s = $os[$a]['matchstring'];
		if (isset($os[$a]['match']))
			$s = $os[$a]['match'];
		if (stristr($text, $s)) {
			return $os[$a]['name'];
		}
	}
	return false;
}

function discover_debug($text) {
	global $debug;
	if ($debug)	print $text;
}

function discover_calculate_start($hosts, $arg) {
	if (!isset($hosts[$arg])) return false;
	$h = trim($hosts[$arg]);

	// 10.1.0.1
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/", $h)) {
		return $h;
	}

	// 10.1.0.*
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.\*$/", $h)) {
		return substr($h,0,-1) . "1";
	}

	// 10.1.0.0/24
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.0\/[0-9]{1,2}$/", $h)) {
		preg_match_all("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\./", $h, $matches);
		return $matches[0][0] . "1";
	}

	// 10.1.0./24
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.\/[0-9]{1,2}$/", $h)) {
		preg_match_all("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\./", $h, $matches);
		return $matches[0][0] . "1";
	}

	// 10.1.0/24
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\/[0-9]{1,2}$/", $h)) {
		preg_match_all("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}/", $h, $matches);
		return $matches[0][0] . ".1";
	}

	// 10.1.0.0/255.255.255.0
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.0\/255\.255\.[0-9]{1,3}\.0$/", $h)) {
		preg_match_all("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\./", $h, $matches);
		return $matches[0][0] . "1";
	}

	// 10.1.0.19/255.255.255.240
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\/255\.255\.255\.[0-9]{1,3}\$/", $h)) {
		preg_match_all("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}/", $h, $matches);
		return $matches[0][0];
	}

	// 10.1.0.19-10.1.0.27
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}-[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\$/", $h)) {
		preg_match_all("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}-/", $h, $matches);
		return substr($matches[0][0], 0, -1);
	}
	discover_debug("  Could not calculate starting address!\n");
	return false;
}

function discover_calculate_total_ips($hosts, $arg) {
	if (!isset($hosts[$arg])) return false;
	$h = trim($hosts[$arg]);

	// 10.1.0.1
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/", $h)) {
		return 1;
	}

	// 10.1.0.*
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.\*$/", $h)) {
		return "254";
	}

	// 10.1.0.0/24
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.0\/[0-9]{1,2}$/", $h)) {
		preg_match_all("/\/[0-9]{1,2}$/", $h, $matches);
		$subnet = substr($matches[0][0], 1);
		$total = 1;
		for ($x = 0; $x < (32-$subnet); $x++) {
			$total = $total * 2;
		}
		return $total-2;
	}

	// 10.1.0./24
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.\/[0-9]{1,2}$/", $h)) {
		preg_match_all("/\/[0-9]{1,2}$/", $h, $matches);
		$subnet = substr($matches[0][0], 1);
		$total = 1;
		for ($x = 0; $x < (32-$subnet); $x++) {
			$total = $total * 2;
		}
		return $total-2;
	}

	// 10.1.0/24
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\/[0-9]{1,2}$/", $h)) {
		preg_match_all("/\/[0-9]{1,2}$/", $h, $matches);
		$subnet = substr($matches[0][0], 1);
		$total = 1;
		for ($x = 0; $x < (32-$subnet); $x++) {
			$total = $total * 2;
		}
		return $total-2;
	}

	// 10.1.0.0/255.255.255.0
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.0\/255\.255\.[0-9]{1,3}\.0$/", $h)) {
		preg_match_all("/[0-9]{1,3}.0$/", $h, $matches);
		$subnet = substr($matches[0][0], 0, -2);
		return ((256 - $subnet)*256) - 2;
	}

	// 10.1.0.19/255.255.255.240
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\/255\.255\.255\.[0-9]{1,3}\$/", $h)) {
		preg_match_all("/\.[0-9]{1,3}$/", $h, $matches);
		$subnet = substr($matches[0][0], 1);
		return (256 - $subnet)-1;
	}

	// 10.1.0.19-10.1.0.27
	if (preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}-[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\$/", $h)) {
		preg_match_all("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}-/", $h, $matches);
		preg_match_all("/-[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/", $h, $matches2);
		$start = substr($matches[0][0], 0, -1);
		$end = substr($matches2[0][0], 1);
		$starta = explode('.', $start);
		$enda = explode('.', $end);
		$start = ($starta[0] * 16777216) + ($starta[1] * 65536) + ($starta[2] * 256) + $starta[3];
		$end = ($enda[0] * 16777216) + ($enda[1] * 65536) + ($enda[2] * 256) + $enda[3];
		return $end - $start + 1;
	}
	discover_debug("  Could not calculate total IPs!\n");
	return false;
}

function discover_get_next_host ($start, $total, $count) {
	if ($count == $total || $total < 1)
		return false;
	$ip = explode('.', $start);

	$a = intval($count/16777216);
	$count = $count - ($a*256);
	$b = intval($count/65536);
	$count = $count - ($b*256);
	$c = intval($count/256);
	$count = $count - ($c*256);
	$d = $count;

	$ip[0] = $ip[0] + $a;
	$ip[1] = $ip[1] + $b;
	$ip[2] = $ip[2] + $c;
	$ip[3] = $ip[3] + $d;
	return implode('.',$ip);
}

function discover_valid_snmp_device (&$device) {
	/* initialize variable */
	$host_up = FALSE;
	$device["snmp_status"] = HOST_DOWN;
	$device["ping_status"] = 0;

	/* force php to return numeric oid's */
	if (function_exists("snmp_set_oid_numeric_print")) {
		snmp_set_oid_numeric_print(TRUE);
	}

	/* if the first read did not work, loop until found */
	$snmp_sysObjectID = @cacti_snmp_get($device["hostname"], $device["snmp_readstring"],
					".1.3.6.1.2.1.1.2.0", $device["snmp_version"],
					"", "", $device["snmp_port"], $device["snmp_timeout"]);

	$snmp_sysObjectID = str_replace("enterprises", ".1.3.6.1.4.1", $snmp_sysObjectID);
	$snmp_sysObjectID = str_replace("OID: ", "", $snmp_sysObjectID);
	$snmp_sysObjectID = str_replace(".iso", ".1", $snmp_sysObjectID);

	if ((strlen($snmp_sysObjectID) > 0) &&
		(!substr_count($snmp_sysObjectID, "No Such Object")) &&
		(!substr_count($snmp_sysObjectID, "Error In"))) {
		$snmp_sysObjectID = trim(str_replace("\"","", $snmp_sysObjectID));
		$host_up = TRUE;
		$device["snmp_status"] = HOST_UP;
	} else if ($device["snmp_readstrings"] != '') {
		/* loop through the default and then other common for the correct answer */
		$read_strings = explode(":",$device["snmp_readstrings"]);

		foreach($read_strings as $snmp_readstring) {
			$snmp_sysObjectID = @cacti_snmp_get($device["hostname"], $snmp_readstring,
					".1.3.6.1.2.1.1.2.0", $device["snmp_version"],
					"", "", $device["snmp_port"], $device["snmp_timeout"]);

			$snmp_sysObjectID = str_replace("enterprises", ".1.3.6.1.4.1", $snmp_sysObjectID);
			$snmp_sysObjectID = str_replace("OID: ", "", $snmp_sysObjectID);
			$snmp_sysObjectID = str_replace(".iso", ".1", $snmp_sysObjectID);

			if ((strlen($snmp_sysObjectID) > 0) &&
				(!substr_count($snmp_sysObjectID, "No Such Object")) &&
				(!substr_count($snmp_sysObjectID, "Error In"))) {
				$snmp_sysObjectID = trim(str_replace("\"", "", $snmp_sysObjectID));
				$device["snmp_readstring"] = $snmp_readstring;
				$device["snmp_status"] = HOST_UP;
				$host_up = TRUE;
				break;
			}else{
				$device["snmp_status"] = HOST_DOWN;
				$host_up = FALSE;
			}
		}
	}

	if ($host_up) {
		$device["snmp_sysObjectID"] = $snmp_sysObjectID;
		$device["community"] = $device["snmp_readstring"];
		/* get system name */
		$snmp_sysName = @cacti_snmp_get($device["hostname"], $device["snmp_readstring"],
					".1.3.6.1.2.1.1.5.0", $device["snmp_version"],
					"", "", $device["snmp_port"], $device["snmp_timeout"]);

		if (strlen($snmp_sysName) > 0) {
			$snmp_sysName = trim(strtr($snmp_sysName,"\""," "));
			$device["snmp_sysName"] = $snmp_sysName;
		}

		/* get system location */
		$snmp_sysLocation = @cacti_snmp_get($device["hostname"], $device["snmp_readstring"],
					".1.3.6.1.2.1.1.6.0", $device["snmp_version"],
					"", "", $device["snmp_port"], $device["snmp_timeout"]);

		if (strlen($snmp_sysLocation) > 0) {
			$snmp_sysLocation = trim(strtr($snmp_sysLocation,"\""," "));
			$device["snmp_sysLocation"] = $snmp_sysLocation;
		}

		/* get system contact */
		$snmp_sysContact = @cacti_snmp_get($device["hostname"], $device["snmp_readstring"],
					".1.3.6.1.2.1.1.4.0", $device["snmp_version"],
					"", "", $device["snmp_port"], $device["snmp_timeout"]);

		if (strlen($snmp_sysContact) > 0) {
			$snmp_sysContact = trim(strtr($snmp_sysContact,"\""," "));
			$device["snmp_sysContact"] = $snmp_sysContact;
		}

		/* get system description */
		$snmp_sysDescr = @cacti_snmp_get($device["hostname"], $device["snmp_readstring"],
					".1.3.6.1.2.1.1.1.0", $device["snmp_version"],
					"", "", $device["snmp_port"], $device["snmp_timeout"]);

		if (strlen($snmp_sysDescr) > 0) {
			$snmp_sysDescr = trim(strtr($snmp_sysDescr,"\""," "));
			$device["snmp_sysDescr"] = $snmp_sysDescr;
		}

		/* get system uptime */
		$snmp_sysUptime = @cacti_snmp_get($device["hostname"], $device["snmp_readstring"],
					".1.3.6.1.2.1.1.3.0", $device["snmp_version"],
					"", "", $device["snmp_port"], $device["snmp_timeout"]);

		if (strlen($snmp_sysUptime) > 0) {
			$snmp_sysUptime = trim(strtr($snmp_sysUptime,"\""," "));
			$device["snmp_sysUptime"] = $snmp_sysUptime;
		}
	}

	return $host_up;
}

/*	gethostbyaddr_wtimeout - This function provides a good method of performing
  a rapid lookup of a DNS entry for a host so long as you don't have to look far.
*/
function discover_get_dns_from_ip($ip, $dns, $timeout = 1000) {
	/* random transaction number (for routers etc to get the reply back) */
	$data = rand(10, 99);

	/* trim it to 2 bytes */
	$data = substr($data, 0, 2);

	/* create request header */
	$data .= "\1\0\0\1\0\0\0\0\0\0";

	/* split IP into octets */
	$octets = explode(".", $ip);

	/* perform a quick error check */
	if (count($octets) != 4) return "ERROR";

	/* needs a byte to indicate the length of each segment of the request */
	for ($x=3; $x>=0; $x--) {
		switch (strlen($octets[$x])) {
		case 1: // 1 byte long segment
			$data .= "\1"; break;
		case 2: // 2 byte long segment
			$data .= "\2"; break;
		case 3: // 3 byte long segment
			$data .= "\3"; break;
		default: // segment is too big, invalid IP
			return "ERROR";
		}

		/* and the segment itself */
		$data .= $octets[$x];
	}

	/* and the final bit of the request */
	$data .= "\7in-addr\4arpa\0\0\x0C\0\1";

	/* create UDP socket */
	$handle = @fsockopen("udp://$dns", 53);

	@stream_set_timeout($handle, floor($timeout/1000), ($timeout*1000)%1000000);
	@stream_set_blocking($handle, 1);

	/* send our request (and store request size so we can cheat later) */
	$requestsize = @fwrite($handle, $data);

	/* get the response */
	$response = @fread($handle, 1000);

	/* check to see if it timed out */
	$info = @stream_get_meta_data($handle);

	/* close the socket */
	@fclose($handle);

	if ($info["timed_out"]) {
		return "timed_out";
	}

	/* more error handling */
	if ($response == "") { return $ip; }

	/* parse the response and find the response type */
	$type = @unpack("s", substr($response, $requestsize+2));

	if ($type[1] == 0x0C00) {
		/* set up our variables */
		$host = "";
		$len = 0;

		/* set our pointer at the beginning of the hostname uses the request
		   size from earlier rather than work it out.
		*/
		$position = $requestsize + 12;

		/* reconstruct the hostname */
		do {
			/* get segment size */
			$len = unpack("c", substr($response, $position));

			/* null terminated string, so length 0 = finished */
			if ($len[1] == 0) {
				/* return the hostname, without the trailing '.' */
				return strtoupper(substr($host, 0, strlen($host) -1));
			}

			/* add the next segment to our host */
			$host .= substr($response, $position+1, $len[1]) . ".";

			/* move pointer on to the next segment */
			$position += $len[1] + 1;
		} while ($len != 0);

		/* error - return the hostname we constructed (without the . on the end) */
		return strtoupper($ip);
	}

	/* error - return the hostname */
	return strtoupper($ip);
}

/*	display_help - displays the usage of the function */
function display_help () {
	print "Network Discovery v1.0, Copyright 2006 - Jimmy Conner\n\n";
	print "usage: findhosts.php [-d] [-h] [--help] [-v] [--version]\n\n";
	print "-f            - Force the execution of a discovery process\n";
	print "-d            - Display verbose output during execution\n";
	print "-r            - Drop and Recreate the Discover Plugin's tables before running\n";
	print "-v --version  - Display this help message\n";
	print "-h --help     - display this help message\n";
}
?>